// LOG; cambio pos bolas para el lado
// Ubique el grafico principal en relacion al diseno original

// Integratos made 
FloatTable data; 
float dataMin, dataMax; 

float plotX1, plotY1; 
float plotX2, plotY2; 
float labelX, labelY;

int rowCount;
int columnCount;
int currentColumn = 0;

int yearMin, yearMax; 
int[] years; 

int yearInterval = 2;
int volumeInterval = 2;
int volumeIntervalMinor = 2; //SMALLL MARKS ON VERTICAL AXIS

float[] tabLeft, tabRight;
float tabTop, tabBottom;
float tabPad = 10;

Integrator[] interpolators; 

PFont plotFont;

color myCol1, myCol2, myCol3;


void setup() { 
  size(1200, 800); 
colorMode(RGB);
  myCol1 = color(200, 0, 0);
  myCol2= color(100, 100, 0);
  myCol2= color(0, 100, 100);

  data = new FloatTable("treeoflife.tsv"); 
  rowCount = data.getRowCount(); 
  columnCount = data.getColumnCount(); 

  years = int(data.getRowNames()); 
  yearMin = years[0]; 
  yearMax = years[years.length - 1]; 

  dataMin = 0; //data.getTableMin();
  dataMax = ceil(data.getTableMax() / volumeInterval) * volumeInterval;
  
  interpolators = new Integrator[rowCount];
  for (int row = 0; row < rowCount; row++) {
    float initialValue = data.getFloat(row, 0);
    interpolators[row] = new Integrator(initialValue);
    interpolators[row].attraction = 0.1;  // Set lower than the default
  }

  
  // Corners of the plotted time series 
  plotX1 = 250; // Distance from left 
  plotX2 = width - 200; // Distance from left 
  labelX = 50;
  plotY1 = 50; //Distance from top 
  plotY2 = height - 50; // Distance from the bottom 
  labelY = height - 25;

  plotFont = createFont("SanSerif", 20);
  textFont(plotFont);

  smooth();
}


void draw() { 
  background(252, 245, 200);
  
  // Show the plot area as a white box 
  fill(255, 255, 220); // BG AREA GRAPH 
  rectMode(CORNERS); 
  noStroke(); 
  rect(plotX1, plotY1, plotX2, plotY2); // BG
  // separation lines
  fill(20);
  rect(plotX1-30, plotY1, plotX1-28, plotY2); // Line left
  rect(plotX2+16, plotY1, plotX2+14, plotY2); // Line left

  drawTitleTabs();
  drawAxisLabels();

   for (int row = 0; row < rowCount; row++) {
    interpolators[row].update();
  }

  drawYearLabels();
  drawVolumeLabels();

  noStroke();
//  fill(#5679C1); 
  drawDataArea(currentColumn);
  drawDataHighlight(currentColumn);
} 


void drawTitleTabs() { // titulos de las paletas de arriba
  rectMode(CORNERS);
  noStroke();
  textSize(20); 
  textAlign(LEFT); 

  //Allocate space in array
  if (tabLeft == null) {
    tabLeft = new float[columnCount];
    tabRight = new float[columnCount];
  }

  float runningX = plotX1; 
  tabTop = plotY1 - textAscent() - 15;
  tabBottom = plotY1;

  for (int col = 0; col < columnCount; col++) {
    String title = data.getColumnName(col);
    tabLeft[col] = runningX;
    float titleWidth = textWidth(title);
    tabRight[col] = tabLeft[col] + tabPad + titleWidth + tabPad;

    //If current tab is white, otherwhise pale gray
    fill(col == currentColumn ? 255 : 224);
    rect(tabLeft[col], tabTop, tabRight[col], tabBottom);

    // if text is black, other wise use darkgray
    fill(col == currentColumn ? 0 : 64);
    text(title, runningX + tabPad, plotY1 - 10);

    runningX = tabRight[col];
  }
}


void mousePressed() { 
  if (mouseY > tabTop && mouseY < tabBottom) {
    for (int col = 0; col < columnCount; col++) { 
      if (mouseX > tabLeft[col] && mouseX < tabRight[col]) {
        setCurrent(col);
      }
    }
  }
}


void setCurrent(int col) {
  currentColumn = col;
  
  for(int row = 0; row < rowCount; row++){
    interpolators[row].target(data.getFloat(row, col));
  }
}


void drawAxisLabels() { // Textos de los ejes
    fill(0);
    textSize(14);
    textLeading(15);
    
    textAlign(CENTER, CENTER);
    text("Galones\nConsumed\nPercapita", labelX, (plotY1+plotY2)/2);
    textAlign(CENTER);
    text("AÑO", (plotX1+plotX2)/2, labelY);
  }

void drawYearLabels() {
    fill(0);
    textSize(10);
    textAlign(CENTER);

    stroke(250);
    strokeWeight(1);

    for (int row = 0; row < rowCount; row++) {
      if (years[row] % yearInterval == 0) {
//        float x = map(years[row], yearMin, yearMax, plotX1, plotX2);
//        text(years[row], x, plotY2 + 10);
//        line(x, plotY1, x, plotY2);
        float x = map(years[row], yearMin, yearMax, plotX1, plotX2);
        text(years[row], x, plotY2 + 10);
        line(x, plotY1, x, plotY2);
        ellipse(x, plotY1, 10, 10);
      }
    }
  }

  // PAGE 66
  void drawVolumeLabels() { // Valores eje vertical
    fill(0);
    textSize(10);
    textAlign(RIGHT);
    stroke(128);
    strokeWeight(1);
    //PAGE 67
    for (float v = dataMin; v <= dataMax; v += volumeIntervalMinor) {
      if (v % volumeIntervalMinor == 0) {
        float y = map(v, dataMin, dataMax, plotY2, plotY1);
        if (v % volumeInterval == 0) {
          float textOffset = textAscent()/2; // Center vertically
          if (v == dataMin) { 
          textOffset = 0;  // Align by the bottom
          } else if (v == dataMax) { 
            textOffset = textAscent(); // Align by the bottom
          }
          text(floor(v), plotX1 - 10, y + textOffset);
          line(plotX1 - 4, y, plotX1, y); //HORIZONTAL LINES IN Y AXIS
        }else {
          // line(plotX1 - 2, y, plotX1, y); //HORIZONTAL LINES IN Y AXIS
        }
      }
    }
  }


void drawDataArea(int col) { // Dibuja el area del grafico
  beginShape();
  for (int row = 0; row < rowCount; row++) {
    if (data.isValid(row, col)) {
      fill((col+1)*70, (col+1)*50, (col+1)*20); //to color areas
      float value = interpolators[row].value;
      float x = map(years[row], yearMin, yearMax, plotX1, plotX2);
      float y = map(value, dataMin, dataMax, plotY2, plotY1);
      vertex(x, y);
    }
  }
  vertex(plotX2, plotY2);
  vertex(plotX1, plotY2);
  endShape(CLOSE);
}


// Representaiton of ellipses 
void drawDataHighlight(int col) {
    for (int row = 0; row < rowCount; row++) {
      if (data.isValid(row, col)) {
        float value = data.getFloat(row, col); 
        float x = map(years[row], yearMin, yearMax, plotX1, plotX2); 
        float y = map(value, dataMin, dataMax, plotY2, plotY1); 

// For ellipses
        float maxvalue = 120;
        float mydiam = map(value, dataMin, dataMax, 10, maxvalue); // diam od ellipses       
        float myx = width-(plotX1/2.5); // ellipses positions
        float myy = plotY1+50; // Pos Y circle 

//        if (dist(mouseX, mouseY, x, y) < 8) {
          if ((mouseX >= x-10) && (mouseX <= x+10) ) { //Pos mouse to reas values in specific pos for ball
                    
          fill(0); 
          textSize(10); 
          textAlign(CENTER); 
          text(nf(value, 0, 2) + " (" + years[row] + ")", x, y-8);
//          strokeWeight(30); 
          point(x, y); 
          fill(255); 
    
          ellipse(myx, myy, maxvalue, maxvalue); // collorbackground ellipse 
          fill((col+1)*70, (col+1)*50, (col+1)*20); //to color areas
          ellipse(myx, myy, mydiam, mydiam); // collor ellipse 01
          text(nf(value, 0, 2) + " (" + years[row] + ")", myx, myy+75); // value of column for ellipse
        }
// STROKE For back od ellipses
//        noFill();
//        stroke(20);
//        strokeWeight(10);
//        ellipse(myx, myy, 100, 100); // collorbackground ellipse 

      }
    }
  }



