import processing.core.*; 

import java.applet.*; 
import java.awt.Dimension; 
import java.awt.Frame; 
import java.awt.event.MouseEvent; 
import java.awt.event.KeyEvent; 
import java.awt.event.FocusEvent; 
import java.awt.Image; 
import java.io.*; 
import java.net.*; 
import java.text.*; 
import java.util.*; 
import java.util.zip.*; 
import java.util.regex.*; 

public class o5_cclab_rvega_frytimeserie extends PApplet {

// first line of the file should be the column headers
// first column should be the row titles
// all other values are expected to be floats
// getFloat(0, 0) returns the first data value in the upper lefthand corner
// files should be saved as "text, tab-delimited"
// empty rows are ignored
// extra whitespace is ignored


class FloatTable {
  int rowCount;
  int columnCount;
  float[][] data;
  String[] rowNames;
  String[] columnNames;
  
  
  FloatTable(String filename) {
    String[] rows = loadStrings(filename);
    
    String[] columns = split(rows[0], TAB);
    columnNames = subset(columns, 1); // upper-left corner ignored
    scrubQuotes(columnNames);
    columnCount = columnNames.length;

    rowNames = new String[rows.length-1];
    data = new float[rows.length-1][];

    // start reading at row 1, because the first row was only the column headers
    for (int i = 1; i < rows.length; i++) {
      if (trim(rows[i]).length() == 0) {
        continue; // skip empty rows
      }
      if (rows[i].startsWith("#")) {
        continue;  // skip comment lines
      }

      // split the row on the tabs
      String[] pieces = split(rows[i], TAB);
      scrubQuotes(pieces);
      
      // copy row title
      rowNames[rowCount] = pieces[0];
      // copy data into the table starting at pieces[1]
      data[rowCount] = parseFloat(subset(pieces, 1));

      // increment the number of valid rows found so far
      rowCount++;      
    }
    // resize the 'data' array as necessary
    data = (float[][]) subset(data, 0, rowCount);
  }
  
  
  public void scrubQuotes(String[] array) {
    for (int i = 0; i < array.length; i++) {
      if (array[i].length() > 2) {
        // remove quotes at start and end, if present
        if (array[i].startsWith("\"") && array[i].endsWith("\"")) {
          array[i] = array[i].substring(1, array[i].length() - 1);
        }
      }
      // make double quotes into single quotes
      array[i] = array[i].replaceAll("\"\"", "\"");
    }
  }
  
  
  public int getRowCount() {
    return rowCount;
  }
  
  
  public String getRowName(int rowIndex) {
    return rowNames[rowIndex];
  }
  
  
  public String[] getRowNames() {
    return rowNames;
  }

  
  // Find a row by its name, returns -1 if no row found. 
  // This will return the index of the first row with this name.
  // A more efficient version of this function would put row names
  // into a Hashtable (or HashMap) that would map to an integer for the row.
  public int getRowIndex(String name) {
    for (int i = 0; i < rowCount; i++) {
      if (rowNames[i].equals(name)) {
        return i;
      }
    }
    //println("No row named '" + name + "' was found");
    return -1;
  }
  
  
  // technically, this only returns the number of columns 
  // in the very first row (which will be most accurate)
  public int getColumnCount() {
    return columnCount;
  }
  
  
  public String getColumnName(int colIndex) {
    return columnNames[colIndex];
  }
  
  
  public String[] getColumnNames() {
    return columnNames;
  }


  public float getFloat(int rowIndex, int col) {
    // Remove the 'training wheels' section for greater efficiency
    // It's included here to provide more useful error messages
    
    // begin training wheels
    if ((rowIndex < 0) || (rowIndex >= data.length)) {
      throw new RuntimeException("There is no row " + rowIndex);
    }
    if ((col < 0) || (col >= data[rowIndex].length)) {
      throw new RuntimeException("Row " + rowIndex + " does not have a column " + col);
    }
    // end training wheels
    
    return data[rowIndex][col];
  }
  
  
  public boolean isValid(int row, int col) {
    if (row < 0) return false;
    if (row >= rowCount) return false;
    //if (col >= columnCount) return false;
    if (col >= data[row].length) return false;
    if (col < 0) return false;
    return !Float.isNaN(data[row][col]);
  }


  public float getColumnMin(int col) {
    float m = Float.MAX_VALUE;
    for (int row = 0; row < rowCount; row++) {
      if (isValid(row, col)) {
        if (data[row][col] < m) {
          m = data[row][col];
        }
      }
    }
    return m;
  }


  public float getColumnMax(int col) {
    float m = -Float.MAX_VALUE;
    for (int row = 0; row < rowCount; row++) {
      if (isValid(row, col)) {
        if (data[row][col] > m) {
          m = data[row][col];
        }
      }
    }
    return m;
  }

  
  public float getRowMin(int row) {
    float m = Float.MAX_VALUE;
    for (int col = 0; col < columnCount; col++) {
      if (isValid(row, col)) {
        if (data[row][col] < m) {
          m = data[row][col];
        }
      }
    }
    return m;
  } 


  public float getRowMax(int row) {
    float m = -Float.MAX_VALUE;
    for (int col = 0; col < columnCount; col++) {
      if (isValid(row, col)) {
        if (data[row][col] > m) {
          m = data[row][col];
        }
      }
    }
    return m;
  }


  public float getTableMin() {
    float m = Float.MAX_VALUE;
    for (int row = 0; row < rowCount; row++) {
      for (int col = 0; col < columnCount; col++) {
        if (isValid(row, col)) {
          if (data[row][col] < m) {
            m = data[row][col];
          }
        }
      }
    }
    return m;
  }


  public float getTableMax() {
    float m = -Float.MAX_VALUE;
    for (int row = 0; row < rowCount; row++) {
      for (int col = 0; col < columnCount; col++) {
        if (isValid(row, col)) {
          if (data[row][col] > m) {
            m = data[row][col];
          }
        }
      }
    }
    return m;
  }
}
// LOG; cambio pos bolas para el lado
// Ubique el grafico principal en relacion al diseno original

// Integratos made 
FloatTable data; 
float dataMin, dataMax; 

float plotX1, plotY1; 
float plotX2, plotY2; 
float labelX, labelY;

int rowCount;
int columnCount;
int currentColumn = 0;

int yearMin, yearMax; 
int[] years; 

int yearInterval = 2;
int volumeInterval = 2;
int volumeIntervalMinor = 2; //SMALLL MARKS ON VERTICAL AXIS

float[] tabLeft, tabRight;
float tabTop, tabBottom;
float tabPad = 10;

Integrator[] interpolators; 

PFont plotFont;

int myCol1, myCol2, myCol3;


public void setup() { 
  size(1200, 800); 
colorMode(RGB);
  myCol1 = color(200, 0, 0);
  myCol2= color(100, 100, 0);
  myCol2= color(0, 100, 100);

  data = new FloatTable("treeoflife.tsv"); 
  rowCount = data.getRowCount(); 
  columnCount = data.getColumnCount(); 

  years = PApplet.parseInt(data.getRowNames()); 
  yearMin = years[0]; 
  yearMax = years[years.length - 1]; 

  dataMin = 0; //data.getTableMin();
  dataMax = ceil(data.getTableMax() / volumeInterval) * volumeInterval;
  
  interpolators = new Integrator[rowCount];
  for (int row = 0; row < rowCount; row++) {
    float initialValue = data.getFloat(row, 0);
    interpolators[row] = new Integrator(initialValue);
    interpolators[row].attraction = 0.1f;  // Set lower than the default
  }

  
  // Corners of the plotted time series 
  plotX1 = 250; // Distance from left 
  plotX2 = width - 200; // Distance from left 
  labelX = 50;
  plotY1 = 50; //Distance from top 
  plotY2 = height - 50; // Distance from the bottom 
  labelY = height - 25;

  plotFont = createFont("SanSerif", 20);
  textFont(plotFont);

  smooth();
}


public void draw() { 
  background(252, 245, 200);
  
  // Show the plot area as a white box 
  fill(255, 255, 220); // BG AREA GRAPH 
  rectMode(CORNERS); 
  noStroke(); 
  rect(plotX1, plotY1, plotX2, plotY2); // BG
  // separation lines
  fill(20);
  rect(plotX1-30, plotY1, plotX1-28, plotY2); // Line left
  rect(plotX2+16, plotY1, plotX2+14, plotY2); // Line left

  drawTitleTabs();
  drawAxisLabels();

   for (int row = 0; row < rowCount; row++) {
    interpolators[row].update();
  }

  drawYearLabels();
  drawVolumeLabels();

  noStroke();
//  fill(#5679C1); 
  drawDataArea(currentColumn);
  drawDataHighlight(currentColumn);
} 


public void drawTitleTabs() { // titulos de las paletas de arriba
  rectMode(CORNERS);
  noStroke();
  textSize(20); 
  textAlign(LEFT); 

  //Allocate space in array
  if (tabLeft == null) {
    tabLeft = new float[columnCount];
    tabRight = new float[columnCount];
  }

  float runningX = plotX1; 
  tabTop = plotY1 - textAscent() - 15;
  tabBottom = plotY1;

  for (int col = 0; col < columnCount; col++) {
    String title = data.getColumnName(col);
    tabLeft[col] = runningX;
    float titleWidth = textWidth(title);
    tabRight[col] = tabLeft[col] + tabPad + titleWidth + tabPad;

    //If current tab is white, otherwhise pale gray
    fill(col == currentColumn ? 255 : 224);
    rect(tabLeft[col], tabTop, tabRight[col], tabBottom);

    // if text is black, other wise use darkgray
    fill(col == currentColumn ? 0 : 64);
    text(title, runningX + tabPad, plotY1 - 10);

    runningX = tabRight[col];
  }
}


public void mousePressed() { 
  if (mouseY > tabTop && mouseY < tabBottom) {
    for (int col = 0; col < columnCount; col++) { 
      if (mouseX > tabLeft[col] && mouseX < tabRight[col]) {
        setCurrent(col);
      }
    }
  }
}


public void setCurrent(int col) {
  currentColumn = col;
  
  for(int row = 0; row < rowCount; row++){
    interpolators[row].target(data.getFloat(row, col));
  }
}


public void drawAxisLabels() { // Textos de los ejes
    fill(0);
    textSize(14);
    textLeading(15);
    
    textAlign(CENTER, CENTER);
    text("Galones\nConsumed\nPercapita", labelX, (plotY1+plotY2)/2);
    textAlign(CENTER);
    text("A\u00d1O", (plotX1+plotX2)/2, labelY);
  }

public void drawYearLabels() {
    fill(0);
    textSize(10);
    textAlign(CENTER);

    stroke(250);
    strokeWeight(1);

    for (int row = 0; row < rowCount; row++) {
      if (years[row] % yearInterval == 0) {
//        float x = map(years[row], yearMin, yearMax, plotX1, plotX2);
//        text(years[row], x, plotY2 + 10);
//        line(x, plotY1, x, plotY2);
        float x = map(years[row], yearMin, yearMax, plotX1, plotX2);
        text(years[row], x, plotY2 + 10);
        line(x, plotY1, x, plotY2);
        ellipse(x, plotY1, 10, 10);
      }
    }
  }

  // PAGE 66
  public void drawVolumeLabels() { // Valores eje vertical
    fill(0);
    textSize(10);
    textAlign(RIGHT);
    stroke(128);
    strokeWeight(1);
    //PAGE 67
    for (float v = dataMin; v <= dataMax; v += volumeIntervalMinor) {
      if (v % volumeIntervalMinor == 0) {
        float y = map(v, dataMin, dataMax, plotY2, plotY1);
        if (v % volumeInterval == 0) {
          float textOffset = textAscent()/2; // Center vertically
          if (v == dataMin) { 
          textOffset = 0;  // Align by the bottom
          } else if (v == dataMax) { 
            textOffset = textAscent(); // Align by the bottom
          }
          text(floor(v), plotX1 - 10, y + textOffset);
          line(plotX1 - 4, y, plotX1, y); //HORIZONTAL LINES IN Y AXIS
        }else {
          // line(plotX1 - 2, y, plotX1, y); //HORIZONTAL LINES IN Y AXIS
        }
      }
    }
  }


public void drawDataArea(int col) { // Dibuja el area del grafico
  beginShape();
  for (int row = 0; row < rowCount; row++) {
    if (data.isValid(row, col)) {
      fill((col+1)*70, (col+1)*50, (col+1)*20); //to color areas
      float value = interpolators[row].value;
      float x = map(years[row], yearMin, yearMax, plotX1, plotX2);
      float y = map(value, dataMin, dataMax, plotY2, plotY1);
      vertex(x, y);
    }
  }
  vertex(plotX2, plotY2);
  vertex(plotX1, plotY2);
  endShape(CLOSE);
}


// Representaiton of ellipses 
public void drawDataHighlight(int col) {
    for (int row = 0; row < rowCount; row++) {
      if (data.isValid(row, col)) {
        float value = data.getFloat(row, col); 
        float x = map(years[row], yearMin, yearMax, plotX1, plotX2); 
        float y = map(value, dataMin, dataMax, plotY2, plotY1); 

// For ellipses
        float maxvalue = 120;
        float mydiam = map(value, dataMin, dataMax, 10, maxvalue); // diam od ellipses       
        float myx = width-(plotX1/2.5f); // ellipses positions
        float myy = plotY1+50; // Pos Y circle 

//        if (dist(mouseX, mouseY, x, y) < 8) {
          if ((mouseX >= x-10) && (mouseX <= x+10) ) { //Pos mouse to reas values in specific pos for ball
                    
          fill(0); 
          textSize(10); 
          textAlign(CENTER); 
          text(nf(value, 0, 2) + " (" + years[row] + ")", x, y-8);
//          strokeWeight(30); 
          point(x, y); 
          fill(255); 
    
          ellipse(myx, myy, maxvalue, maxvalue); // collorbackground ellipse 
          fill((col+1)*70, (col+1)*50, (col+1)*20); //to color areas
          ellipse(myx, myy, mydiam, mydiam); // collor ellipse 01
          text(nf(value, 0, 2) + " (" + years[row] + ")", myx, myy+75); // value of column for ellipse
        }
// STROKE For back od ellipses
//        noFill();
//        stroke(20);
//        strokeWeight(10);
//        ellipse(myx, myy, 100, 100); // collorbackground ellipse 

      }
    }
  }




class Integrator {

  final float DAMPING = 0.5f;
  final float ATTRACTION = 0.2f;

  float value;
  float vel;
  float accel;
  float force;
  float mass = 1;

  float damping = DAMPING;
  float attraction = ATTRACTION;
  boolean targeting;
  float target;

  Integrator() {
  }

  Integrator(float value) {
    this.value = value;
  }

  Integrator(float value, float damping, float attraction) {
    this.value = value;
    this.damping = damping;
    this.attraction = attraction;
  }

  public void set(float v) {
    value = v;
  }

  public void update() {
    if (targeting) {
      force += attraction * (target - value);
    }

    accel = force / mass;
    vel = (vel + accel) * damping;
    value += vel;

    force = 0;
  }

  public void target(float t) {
    targeting = true;
    target = t;
  }

  public void noTarget() {
    targeting = false;
  }
}

  static public void main(String args[]) {
    PApplet.main(new String[] { "--bgcolor=#C0C0C0", "o5_cclab_rvega_frytimeserie" });
  }
}
